//+-------------------------------------------------------------------------------------+
//|                                                            AMKAandWilder_Expert.mq4 |
//|                                                                           Scriptong |
//|                                                                                     |
//+-------------------------------------------------------------------------------------+
#property copyright "Scriptong"
#property link      "scriptong@mail.ru"

//---- input parameters
extern double    Lots        = 0.1;                          //   
extern int       TakeProfit = 2000;
extern int       StopLoss = 2000;
extern string    A3 = "  AMKA";
extern int       AMKAPeriod=9;                                             //  AMKA
extern int       FastEMAPeriod=2;                   //     AMKA
extern int       SlowEMAPeriod=30;                  //     AMKA
extern double    PowLevel=2.0;         // ,     AMKA
                                       // 2 -  , 
                                       // 3 -   . .
extern double    dK = 1.0;                                     //   
extern bool      UseStdDev = True;      //    ?
extern int       AMKAPrice = 5;                              //   AMKA (0 - 6)
extern string    A4 = "=====================================";
extern string    OpenOrderSound = "ok.wav";                  //    
extern int       MagicNumber = 17589 ;                         //   
bool Activate, FreeMarginAlert, FatalError;
double Tick, Spread, StopLevel, MinLot, MaxLot, LotStep, CurRed, CurBlue;
datetime LastBar;
int TypeOrder, LastType, TicketOrder, Signal;


//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
int init()
  {
//----
   Activate = False;
// - 1 - ==      ========================================
   Tick = MarketInfo(Symbol(), MODE_TICKSIZE);                         //  
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                  //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //   
   MinLot = MarketInfo(Symbol(), MODE_MINLOT);    //    
   MaxLot = MarketInfo(Symbol(), MODE_MAXLOT);   //    
   LotStep = MarketInfo(Symbol(), MODE_LOTSTEP);          //    
// - 1 - ==   =============================================================

// - 2 - ==          =======
   Lots = MathRound(Lots/LotStep)*LotStep; //     
   if(Lots < MinLot || Lots > MaxLot) //     MinLot    MaxLot
     {
      Comment(" Lots     !  !");
      return(0);
     }
// - 2 - ==   =============================================================

   LastBar = 0;
   CurRed = 0;
   CurBlue = 0;

   Activate = True; //    ,    
   
//----
   return(0);
  }

//+-------------------------------------------------------------------------------------+
//|                                                       |
//+-------------------------------------------------------------------------------------+
int deinit()
{
//----
   Comment("");
//----
   return(0);
}

//+-------------------------------------------------------------------------------------+
//|                                              |
//+-------------------------------------------------------------------------------------+
double ND(double A)
{
 return(NormalizeDouble(A, Digits));
}  
 
//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
string ErrorToString(int Error)
{
 switch(Error)
   {
    case 2: return("  ,   ."); 
    case 5: return("    ,  ."); 
    case 6: return("   ,   ."); 
    case 64: return(" ,   .");
    case 132: return(" ."); 
    case 133: return(" ."); 
    case 149: return(" ."); 
   }
}
  
//+-------------------------------------------------------------------------------------+
//|                                                                      |
//| :                                                                         |
//|   True -                                                       |
//|   False -                                                             |
//+-------------------------------------------------------------------------------------+
bool OpenOrder(int Type, double Price, double SL, double TP)
{
 //     
 if(AccountFreeMarginCheck(Symbol(), OP_BUY, Lots) <= 0 || GetLastError() == 134) 
  {
   if(!FreeMarginAlert)
    {
     Print("    . Free Margin = ", 
           AccountFreeMargin());
     FreeMarginAlert = True;
    } 
   return(False);  
  }
 FreeMarginAlert = False;  
 // ---------------------------------------------

 switch (Type)
   {
    case OP_BUY: string S = "BUY"; break;
    case OP_SELL: S = "SELL"; break;
    case OP_BUYSTOP: S = "BUYSTOP"; break;
    case OP_SELLSTOP: S = "SELLSTOP"; break;
    case OP_BUYLIMIT: S = "BUYLIMIT"; break;
    case OP_SELLLIMIT: S = "SELLLIMIT"; break;
   }

 if(WaitForTradeContext())  //    
   {  
    Comment("     ", S, " ...");
    int ticket=OrderSend(Symbol(), Type, Lots, Price, 0, 
               SL, TP, NULL, MagicNumber, 0, CLR_NONE);        //  
    //     
    if(ticket<0)
      {
       int Error = GetLastError();
       if(Error == 2 || Error == 5 || Error == 6 || Error == 64 
          || Error == 132 || Error == 133 || Error == 149)     //   
         {
          Comment("     . . "+
                   ErrorToString(Error)+"  !");
          FatalError = True;
         }
        else 
         Comment("   ", S, ": ", Error);       //  
       return(False);
      }
    // ---------------------------------------------
    
    //      
    Comment(" ", S, "  !"); 
    PlaySound(OpenOrderSound); 
    return(True); 
    // ------------------------
   }
  else
   {
    Comment("     !");
    return(False);  
   } 
}

//+-------------------------------------------------------------------------------------+
//|   .   ,   True,  - False    |
//+-------------------------------------------------------------------------------------+
bool WaitForTradeContext()
{
 int P = 0;
 //  ""
 while(IsTradeContextBusy() && P < 5)
   {
    P++;
    Sleep(1000);
   }
 // -------------  
 if(P == 5)
   return(False);
 return(True);    
}

//+-------------------------------------------------------------------------------------+
//|   /                                              |
//+-------------------------------------------------------------------------------------+
void GetSignal(int Bar)
{
 Signal = 0;
// - 1 - ==    AMKA ===================================================
 double RedDot = iCustom(Symbol(), 0, "AMKA", AMKAPeriod, FastEMAPeriod, 
                         SlowEMAPeriod, PowLevel, dK, UseStdDev, AMKAPrice, 1, Bar);
 double BlueDot = iCustom(Symbol(), 0, "AMKA", AMKAPeriod, FastEMAPeriod, 
                          SlowEMAPeriod, PowLevel, dK, UseStdDev, AMKAPrice, 2, Bar);
 if (RedDot != 0 && BlueDot == 0) 
   {
    CurRed = RedDot;
    CurBlue = 0.0;
   } 
 if (BlueDot != 0 && RedDot == 0) 
   {
    CurBlue = BlueDot;
    CurRed = 0.0;
   } 
// - 1 - ==   =============================================================

// - 3 - ==   ==============================================================
  if (CurRed != 0)
    if (ND(Open[Bar]) > ND(Close[Bar]))   //   
      if (ND(Close[Bar]) < ND(Low[Bar+1])) //       
        if (ND(Open[Bar+1]) < ND(Close[Bar+1])) //    
          Signal = 1;
    
  if (CurBlue != 0)
    if (ND(Open[Bar]) < ND(Close[Bar]))   //   
      if (ND(Close[Bar]) > ND(High[Bar+1])) //       
        if (ND(Open[Bar+1]) > ND(Close[Bar+1])) //    
          Signal = 2;
// - 3 - ==   =============================================================
}  

//+-------------------------------------------------------------------------------------+
//|     Type.     ,  1.             |
//|   ,   2.    0.              |
//+-------------------------------------------------------------------------------------+
int CheckOrdersReal(int Type)
{
 for (int i = OrdersTotal()-1; i >= 0; i--)
   if(OrderSelect(i, SELECT_BY_POS))
     if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber 
        && OrderType() < 2)                                       //  "" 
       if(OrderType() == Type)                                  //    Type
         {
          if (WaitForTradeContext())  
            {
             if(Type == OP_BUY)
               double Pr = ND(MarketInfo(Symbol(), MODE_BID));
              else
               Pr = ND(MarketInfo(Symbol(), MODE_ASK));
             if (!OrderClose(OrderTicket(), OrderLots(), Pr, 3))//    
              return(1);                                 //    -  1
            }  
           else
            return(1);// 1,       
         }
        else
         return(2);          //  2,   ,  
 return(0);                                                   //   ,   0
}  

//+-------------------------------------------------------------------------------------+
//|    "if-else"                                                 |
//+-------------------------------------------------------------------------------------+
double IF(bool Condition, double IfTrue, double IfFalse)
{
 if (Condition) return(IfTrue);    //   ,    IfTrue
 else           return(IfFalse);    //   ,    IfFalse
}  

//+-------------------------------------------------------------------------------------+
//|  START                                                               |
//+-------------------------------------------------------------------------------------+
int start()
  {
// - 1 -  ==    ? ===========================================
   if (!Activate || FatalError)             //   ,  
    return(0);           //  init         
// - 1 -  ==   ============================================================

// - 2 -  ==     ==============================================
   if (LastBar == Time[0])                        //      
     return(0);                                                         //  
// - 2 -  ==   ============================================================

// - 3 - ==      ========================================
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                  //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //   
// - 3 -  ==   ============================================================
     
// - 4 - ==    =====================================================
   GetSignal(1);                                                       //   
// - 4 -  ==   ============================================================
   
   double Price = 0;
   int Type = -1;
   
// - 6 - ==    ====================================================
   if (Signal == 1)
     {
      int Res = CheckOrdersReal(OP_SELL);
      RefreshRates();
      if (Res == 0)                                          //    
        {                                               
         Price = ND(Ask);
         double TP = ND(IF(TakeProfit > StopLevel/Point, Price+TakeProfit*Tick,
                           Price + StopLevel));
         double SL = ND(IF(StopLoss > (StopLevel+Spread)/Point, Price-StopLoss*Tick,
                           Price - StopLevel-Spread));
         Type = OP_BUY;
        }                    
      if (Res == 1) return(0);                      //    
     }
// - 6 -  ==   ============================================================
     
// - 7 - ==   SellStop ===================================================
   if (Signal == 2)
     {
      Res = CheckOrdersReal(OP_BUY);
      if (Res == 0)                                          //    
        {               
         RefreshRates();
         Price = ND(Bid);
         TP = ND(IF(TakeProfit > StopLevel/Point, Price-TakeProfit*Tick, 
                    Price - StopLevel));
         SL = ND(IF(StopLoss > (StopLevel+Spread)/Point, Price+StopLoss*Tick,
                    Price + StopLevel + Spread));
         Type = OP_SELL;
        }                    
      if (Res == 1) return(0);                      //    
     }
// - 7 -  ==   ============================================================

   if (Type >= 0)
     if (!OpenOrder(Type, Price, SL, TP))             //     
        return(0);                                     //     
        
   LastBar = Time[0];     

   return(0);
  }
//+------------------------------------------------------------------+